/*  (c) 2008 Benjamin Schroeter
 *  You are allowed to use this code in all your commercial, non-commercial, open and closed source projects.
 *  But you have to leave this Copyright message in all files. This code is provided "as is" without warranty of any kind.
 *  If you have any enhancements or bug reports I will appreciate a short feedback.
 *  For more information visit the article to this code on my blog at 
 *  http://www.ticklishtechs.net/2008/02/17/adding-properties-to-domain-classes-at-runtime-with-dsl-tools/ */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualStudio.Modeling;

namespace BenjaminSchroeter.DynamicDslProperties
{
    public partial class EditDynamicPropertiesDialog : Form
    {
        private readonly DynamicPropertyCollection prop;
        private readonly Store store;

        public EditDynamicPropertiesDialog()
        {
            InitializeComponent();
        }

        public EditDynamicPropertiesDialog(DynamicPropertyCollection prop, Store store) : this()
        {
            this.prop = prop;
            this.store = store;

            FillListBox();
        }

        private void FillListBox()
        {
            this.listBox1.Items.Clear();
            foreach (DynamicProperty property in prop)
                this.listBox1.Items.Add(property);
        }

        private void Close_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void Add_Click(object sender, EventArgs e)
        {
            if ( this.textBoxName.Text == "" )
                return;

            Type type = Type.GetType(this.comboBox1.Text);
            if (type == null)
            {
                MessageBox.Show(string.Format("Cannot find type {0}.", this.comboBox1.Text),"Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
                return;
            }

            if( prop.ExistsProperty(this.textBoxName.Text))
            {
                MessageBox.Show(string.Format("The property '{0}' already exists.", this.textBoxName.Text), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;                
            }

            Transaction t = store.TransactionManager.BeginTransaction("Add property to dynamic property collection.");
            prop.AddProperty(this.textBoxName.Text, type, null);
            t.Commit();

            FillListBox();
        }

        private void Delete_Click(object sender, EventArgs e)
        {
            if ( this.listBox1.SelectedItem == null )
                return;

            DynamicProperty p = (DynamicProperty) this.listBox1.SelectedItem;

            Transaction t = store.TransactionManager.BeginTransaction("Remove property from dynamic property collection");
            prop.RemoveProperty(p.Name);
            t.Commit();

            FillListBox();
        }

        private void listBox1_DoubleClick(object sender, EventArgs e)
        {
            if (this.listBox1.SelectedItem == null)
                return;

            DynamicProperty p = (DynamicProperty)this.listBox1.SelectedItem;
            if ( p.Type == typeof (DynamicPropertyCollection))
            {
                if (p.Value == null)
                    p.Value = new DynamicPropertyCollection();
                EditDynamicPropertiesDialog f = new EditDynamicPropertiesDialog((DynamicPropertyCollection) p.Value, store);
                f.ShowDialog(this);
            }
        }
    }
}
